### This script translates the names in the target dataset (see PU_1_2_spp_cop.csv) 
### to the PASL format (see PASL_with_authors.csv)

# Loading libraries for matching species names
library(dplyr)
library(fuzzyjoin)


## 0. Creating translation table (PASL_with_authors.csv)
#transl = read.csv("./PASL.txt", stringsAsFactors = F, header = F, sep = ",")
#transl = unlist(sapply(c(1:nrow(transl)), function(x) substr(transl[x,], 17, 1000)))
#transl_1 = sapply(transl, function(x) strsplit(x, "  *   ")[[1]][1])
#transl_2 = sapply(transl, function(x) strsplit(x, "  *   ")[[1]][2])
#transl = as.data.frame(cbind(transl_1,transl_2))
#rownames(transl) = NULL
#colnames(transl) = c("from", "to")
#write.csv(transl,"PASL_with_authors.csv", row.names = F)

## 1. Loading dictionary of species with and without authors
transl = read.csv("./PASL_with_authors.csv") 
## removing special characters
transl$from = gsub("\xfb", "o", transl$from)
transl$from = gsub("\xd6", " ", transl$from)
transl$from = gsub("\xa0", " ", transl$from)
transl$from = gsub("\xf6", "o", transl$from)
transl$from = gsub("\xe4", "a", transl$from)
transl$from = gsub("\xe0", " ", transl$from)

## 2. Pred-processing target table - i.e. replacing sp., spp. with species

targ1 = read.csv("./PU_1_2_spp_cop.csv", stringsAsFactors = F, sep = ",")
targ1 = as.data.frame(targ1[,1])
colnames(targ1) = "original"
targ1$original = as.character(targ1$original)
targ1$from = targ1$original

targ1$from = unlist(sapply(c(1:nrow(targ1)), function(x) {
  target = targ1$from[x]
  
  # removing special characters
  target = gsub("\xa0", " ", target)
  target = gsub("\xf6", "o", target)
  target = gsub("\xe4", "a", target)
  
  # replacing sp, spp etc.
  last_3chars = substr(target, nchar(target) - 2, nchar(target))
  last_4chars = substr(target, nchar(target) - 3, nchar(target))
  last_5chars = substr(target, nchar(target) - 4, nchar(target))
  last_6chars = substr(target, nchar(target) - 5, nchar(target))
  if (last_3chars == " sp") {
    target = substr(target, 1, nchar(target) - 3)
    target = paste(target, "species")
  }
  if ((last_4chars == " spp") | (last_4chars == " sp.") | (last_4chars == " sp ")) {
    target = substr(target, 1, nchar(target) - 4)
    target = paste(target, "species")
  }
  if ((last_5chars == " spp ") | (last_5chars == " sp. ") | (last_5chars == " spp.")) {
    target = substr(target, 1, nchar(target) - 5)
    target = paste(target, "species")
  }
  if (last_6chars == " spp. ") {
    target = substr(target, 1, nchar(target) - 6)
    target = paste(target, "species")
  }
  return(target)
}))

## 3. Translating the names in the target dataset to the PASL format

joined = stringdist_inner_join(na.omit(targ1), na.omit(transl),  by = "from", max_dist = 2)
targ1$PASL_name = joined$from.y[match(targ1$from, joined$from.x)]
targ1$PASL_dedy = joined$to[match(targ1$from, joined$from.x)]
summary(targ1)
write.csv(targ1,"./PU_1_2_spp_сop_translations.csv", row.names = F)
